/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package business;

import entities.Conge;
import exceptions.CongeAlreadyExistingException;
import exceptions.CongeNotFoundException;
import exceptions.CongeNotValidException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;

/**
 *
 * @author hardebolle
 */
@Singleton(name = "congesSingleton")
public class CongesSingleton {

    private HashMap<String, Conge> listeConges;

    @PostConstruct
    private void init() {
        this.listeConges = new HashMap<String, Conge>();
        Conge c1 = new Conge("jack", "Doe", "Jack");
        this.listeConges.put(c1.getId(), c1);
        Conge c2 = new Conge("james", "Doe", "James");
        this.listeConges.put(c2.getId(), c2);
        Conge c3 = new Conge("john", "Doe", "John");
        this.listeConges.put(c3.getId(), c3);
        Conge c4 = new Conge("bunny", "Bunny", "Bugs");
        this.listeConges.put(c4.getId(), c4);
        Conge c5 = new Conge("duck", "Duck", "Donald");
        this.listeConges.put(c5.getId(), c5);
        Conge c6 = new Conge("mouse", "Mouse", "Mickey");
        this.listeConges.put(c6.getId(), c6);
    }

    public List<Conge> getListeConges(){
        return new ArrayList<Conge>(listeConges.values());
    }

    public void removeConges(String identifiant) throws CongeNotFoundException {
        if (!this.listeConges.containsKey(identifiant)) {
            throw new CongeNotFoundException(identifiant);
        }
        this.listeConges.remove(identifiant);
    }

    public void addConges(Conge courant) throws CongeAlreadyExistingException {
        if (this.listeConges.containsKey(courant.getId())) {
            throw new CongeAlreadyExistingException(courant.getId());
        }
        this.listeConges.put(courant.getId(), courant);
    }
    
    

    public Integer getSoldeRtt(String identifiant) throws CongeNotFoundException {
        if (!this.listeConges.containsKey(identifiant)) {
            CongeNotFoundException e = new CongeNotFoundException(identifiant);
            throw e;
        }
        return this.listeConges.get(identifiant).getRtt();
    }

    public void setSoldeRtt(String identifiant, Integer nouveauSolde) throws CongeNotFoundException, CongeNotValidException {
        if (!this.listeConges.containsKey(identifiant)) {
            throw new CongeNotFoundException(identifiant);
        }
        this.validateCongesNewValue("rtt", nouveauSolde);

        Conge c = this.listeConges.get(identifiant);
        c.setRtt(nouveauSolde);
    }

    
    public Integer getSoldeCp(String identifiant) throws CongeNotFoundException {
        if (!this.listeConges.containsKey(identifiant)) {
            throw new CongeNotFoundException(identifiant);
        }
        return this.listeConges.get(identifiant).getCp();
    }

    public void setSoldeCp(String identifiant, Integer nouveauSolde) throws CongeNotFoundException, CongeNotValidException {
        if (!this.listeConges.containsKey(identifiant)) {
            throw new CongeNotFoundException(identifiant);
        }
        this.validateCongesNewValue("cp", nouveauSolde);

        Conge c = this.listeConges.get(identifiant);
        c.setCp(nouveauSolde);
    }

    
    
    
    private void validateCongesNewValue(String attributeName, Object newvalue) throws CongeNotValidException {
        ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
        Validator validator = factory.getValidator();

        Set<ConstraintViolation<Conge>> constraintViolations = validator.validateValue(Conge.class, attributeName, newvalue);
        if (constraintViolations.size() > 0) {
            String validationMsg = "";
            for (ConstraintViolation<Conge> contrainte : constraintViolations) {
                validationMsg += contrainte.getMessage();
            }
            throw new CongeNotValidException(validationMsg);
        }
    }
}
